@@ -46,6 +46,7 @@
 #include "brasero-session.h"
 #include "burn-plugin-manager.h"
 #include "burn-image-format.h"
+#include "libbrasero-marshal.h"
 
 #include "brasero-tags.h"
 #include "brasero-track-image.h"
@@ -54,6 +55,16 @@
 #include "brasero-burn-lib.h"
 #include "brasero-session-helper.h"
 
+/**
+ * SECTION:brasero-session-cfg
+ * @short_description: Configure automatically a #BraseroBurnSession object
+ * @see_also: #BraseroBurn, #BraseroBurnSession
+ * @include: brasero-session-cfg.h
+ *
+ * This object configures automatically a session reacting to any change
+ * made to the various parameters.
+ **/
+
 typedef struct _BraseroSessionCfgPrivate BraseroSessionCfgPrivate;
 struct _BraseroSessionCfgPrivate
 {
@@ -88,6 +99,25 @@ G_DEFINE_TYPE (BraseroSessionCfg, brasero_session_cfg, BRASERO_TYPE_SESSION_SPAN
 #define BRASERO_DEST_SAVED_FLAGS		(BRASERO_DRIVE_PROPERTIES_FLAGS|BRASERO_BURN_FLAG_MULTI)
 #define BRASERO_DRIVE_PROPERTIES_KEY		"/apps/brasero/drives"
 
+/**
+ * This is to handle tags (and more particularly video ones)
+ */
+
+static void
+brasero_session_cfg_tag_changed (BraseroBurnSession *session,
+                                 const gchar *tag)
+{
+	if (!strcmp (tag, BRASERO_VCD_TYPE)) {
+		int svcd_type;
+
+		svcd_type = brasero_burn_session_tag_lookup_int (session, BRASERO_VCD_TYPE);
+		if (svcd_type != BRASERO_SVCD)
+			brasero_burn_session_tag_add_int (session,
+			                                  BRASERO_VIDEO_OUTPUT_ASPECT,
+			                                  BRASERO_VIDEO_ASPECT_4_3);
+	}
+}
+
 /**
  * brasero_session_cfg_has_default_output_path:
  * @cfg: a #BraseroSessionCfg
@@ -143,6 +173,8 @@ brasero_session_cfg_set_output_image (BraseroBurnSession *session,
 				      const gchar *toc)
 {
 	gchar *dot;
+	gchar *set_toc = NULL;
+	gchar * set_image = NULL;
 	BraseroBurnResult result;
 	BraseroBurnSessionClass *klass;
 	const gchar *suffixes [] = {".iso",
@@ -151,8 +183,39 @@ brasero_session_cfg_set_output_image (BraseroBurnSession *session,
 				    ".toc",
 				    NULL };
 
-	/* First set all information */
+	/* Make sure something actually changed */
 	klass = BRASERO_BURN_SESSION_CLASS (brasero_session_cfg_parent_class);
+	klass->get_output_path (BRASERO_BURN_SESSION (session),
+	                        &set_image,
+	                        &set_toc);
+
+	if (!set_image && !set_toc) {
+		/* see if image and toc set paths differ */
+		brasero_burn_session_get_output (BRASERO_BURN_SESSION (session),
+		                                 &set_image,
+		                                 &set_toc);
+		if (set_image && image && !strcmp (set_image, image)) {
+			/* It's the same default path so no 
+			 * need to carry on and actually set
+			 * the path of image. */
+			image = NULL;
+		}
+
+		if (set_toc && toc && !strcmp (set_toc, toc)) {
+			/* It's the same default path so no 
+			 * need to carry on and actually set
+			 * the path of image. */
+			toc = NULL;
+		}
+	}
+
+	if (set_image)
+		g_free (set_image);
+
+	if (set_toc)
+		g_free (set_toc);
+
+	/* First set all information */
 	result = klass->set_output_image (session,
 					  format,
 					  image,
@@ -169,40 +232,75 @@ brasero_session_cfg_set_output_image (BraseroBurnSession *session,
 
 	if (format & BRASERO_IMAGE_FORMAT_BIN) {
 		dot = g_utf8_strrchr (image, -1, '.');
-		if (!strcmp (suffixes [0], dot)) {
+		if (strcmp (suffixes [0], dot)) {
 			gboolean res;
 
 			res = brasero_session_cfg_wrong_extension_signal (BRASERO_SESSION_CFG (session));
-			if (res)
-				brasero_image_format_fix_path_extension (format, FALSE, image);
+			if (res) {
+				gchar *fixed_path;
+
+				fixed_path = brasero_image_format_fix_path_extension (format, FALSE, image);
+				/* NOTE: call ourselves with the fixed path as this way,
+				 * in case the path is the same as the default one after
+				 * fixing the extension we'll keep on using default path */
+				result = brasero_burn_session_set_image_output_full (session,
+				                                                     format,
+				                                                     fixed_path,
+				                                                     toc);
+				g_free (fixed_path);
+			}
 		}
 	}
 	else {
 		dot = g_utf8_strrchr (toc, -1, '.');
 
 		if (format & BRASERO_IMAGE_FORMAT_CLONE
-		&& !strcmp (suffixes [1], dot)) {
+		&& strcmp (suffixes [1], dot)) {
 			gboolean res;
 
 			res = brasero_session_cfg_wrong_extension_signal (BRASERO_SESSION_CFG (session));
-			if (res)
-				brasero_image_format_fix_path_extension (format, FALSE, toc);
+			if (res) {
+				gchar *fixed_path;
+
+				fixed_path = brasero_image_format_fix_path_extension (format, FALSE, toc);
+				result = brasero_burn_session_set_image_output_full (session,
+				                                                     format,
+				                                                     image,
+				                                                     fixed_path);
+				g_free (fixed_path);
+			}
 		}
 		else if (format & BRASERO_IMAGE_FORMAT_CUE
-		     && !strcmp (suffixes [2], dot)) {
+		     && strcmp (suffixes [2], dot)) {
 			gboolean res;
 
 			res = brasero_session_cfg_wrong_extension_signal (BRASERO_SESSION_CFG (session));
-			if (res)
-				brasero_image_format_fix_path_extension (format, FALSE, toc);
+			if (res) {
+				gchar *fixed_path;
+
+				fixed_path = brasero_image_format_fix_path_extension (format, FALSE, toc);
+				result = brasero_burn_session_set_image_output_full (session,
+				                                                     format,
+				                                                     image,
+				                                                     fixed_path);
+				g_free (fixed_path);
+			}
 		}
 		else if (format & BRASERO_IMAGE_FORMAT_CDRDAO
-		     && !strcmp (suffixes [3], dot)) {
+		     && strcmp (suffixes [3], dot)) {
 			gboolean res;
 
 			res = brasero_session_cfg_wrong_extension_signal (BRASERO_SESSION_CFG (session));
-			if (res)
-				brasero_image_format_fix_path_extension (format, FALSE, toc);
+			if (res) {
+				gchar *fixed_path;
+
+				fixed_path = brasero_image_format_fix_path_extension (format, FALSE, toc);
+				result = brasero_burn_session_set_image_output_full (session,
+				                                                     format,
+				                                                     image,
+				                                                     fixed_path);
+				g_free (fixed_path);
+			}
 		}
 	}
 
@@ -775,14 +873,7 @@ brasero_session_cfg_check_size (BraseroSessionCfg *self)
 		return BRASERO_SESSION_NO_OUTPUT;
 	}
 
-	flags = brasero_burn_session_get_flags (BRASERO_BURN_SESSION (self));
-	if (flags & (BRASERO_BURN_FLAG_MERGE|BRASERO_BURN_FLAG_APPEND))
-		brasero_medium_get_free_space (medium, NULL, &disc_size);
-	else if (brasero_burn_session_can_blank (BRASERO_BURN_SESSION (self)) == BRASERO_BURN_OK)
-		brasero_medium_get_capacity (medium, NULL, &disc_size);
-	else
-		brasero_medium_get_free_space (medium, NULL, &disc_size);
-
+	disc_size = brasero_burn_session_get_available_medium_space (BRASERO_BURN_SESSION (self));
 	if (disc_size < 0)
 		disc_size = 0;
 
@@ -828,6 +919,7 @@ brasero_session_cfg_check_size (BraseroSessionCfg *self)
 		return BRASERO_SESSION_INSUFFICIENT_SPACE;
 	}
 
+	flags = brasero_burn_session_get_flags (BRASERO_BURN_SESSION (self));
 	if (!(flags & BRASERO_BURN_FLAG_OVERBURN)) {
 		BraseroSessionCfgPrivate *priv;
 
@@ -928,8 +1020,8 @@ brasero_session_cfg_update (BraseroSessionCfg *self,
 			       0);
 		return;
 	}
-	/* it can be empty with an empty track */
 
+	/* it can be empty with just an empty track */
 	if (brasero_track_type_get_has_medium (source)
 	&&  brasero_track_type_get_medium_type (source) == BRASERO_MEDIUM_NONE) {
 		brasero_track_type_free (source);
@@ -1067,6 +1159,16 @@ brasero_session_cfg_update (BraseroSessionCfg *self,
 		return;
 	}
 
+	/* Special case for video projects */
+	if (brasero_track_type_get_has_stream (source)
+	&& BRASERO_STREAM_FORMAT_HAS_VIDEO (brasero_track_type_get_stream_format (source))) {
+		/* Only set if it was not already set */
+		if (brasero_burn_session_tag_lookup (BRASERO_BURN_SESSION (self), BRASERO_VCD_TYPE, NULL) != BRASERO_BURN_OK)
+			brasero_burn_session_tag_add_int (BRASERO_BURN_SESSION (self),
+							  BRASERO_VCD_TYPE,
+							  BRASERO_SVCD);
+	}
+
 	brasero_track_type_free (source);
 
 	/* Configure flags */
@@ -1200,11 +1302,46 @@ brasero_session_cfg_output_changed (BraseroBurnSession *session,
 				    BraseroMedium *former)
 {
 	BraseroSessionCfgPrivate *priv;
+	BraseroTrackType *type;
 
 	priv = BRASERO_SESSION_CFG_PRIVATE (session);
 	if (priv->disabled)
 		return;
 
+	/* Case for video project */
+	type = brasero_track_type_new ();
+	brasero_burn_session_get_input_type (session, type);
+
+	if (brasero_track_type_get_has_stream (type)
+	&&  BRASERO_STREAM_FORMAT_HAS_VIDEO (brasero_track_type_get_stream_format (type))) {
+		BraseroMedia media;
+
+		media = brasero_burn_session_get_dest_media (session);
+		if (media & BRASERO_MEDIUM_DVD)
+			brasero_burn_session_tag_add_int (session,
+			                                  BRASERO_DVD_STREAM_FORMAT,
+			                                  BRASERO_AUDIO_FORMAT_AC3);
+		else if (media & BRASERO_MEDIUM_CD)
+			brasero_burn_session_tag_add_int (session,
+							  BRASERO_DVD_STREAM_FORMAT,
+							  BRASERO_AUDIO_FORMAT_MP2);
+		else {
+			BraseroImageFormat format;
+
+			format = brasero_burn_session_get_output_format (session);
+			if (format == BRASERO_IMAGE_FORMAT_CUE)
+				brasero_burn_session_tag_add_int (session,
+								  BRASERO_DVD_STREAM_FORMAT,
+								  BRASERO_AUDIO_FORMAT_MP2);
+			else
+				brasero_burn_session_tag_add_int (session,
+								  BRASERO_DVD_STREAM_FORMAT,
+								  BRASERO_AUDIO_FORMAT_AC3);
+		}
+	}
+
+	brasero_track_type_free (type);
+
 	brasero_session_cfg_save_drive_properties (BRASERO_SESSION_CFG (session),
 						   former);
 
@@ -1438,6 +1575,7 @@ brasero_session_cfg_class_init (BraseroSessionCfgClass *klass)
 	session_class->track_changed = brasero_session_cfg_track_changed;
 	session_class->output_changed = brasero_session_cfg_output_changed;
 	session_class->flags_changed = brasero_session_cfg_flags_changed;
+	session_class->tag_changed = brasero_session_cfg_tag_changed;
 
 	session_cfg_signals [WRONG_EXTENSION_SIGNAL] =
 		g_signal_new ("wrong_extension",
@@ -1445,8 +1583,8 @@ brasero_session_cfg_class_init (BraseroSessionCfgClass *klass)
 		              G_SIGNAL_RUN_LAST | G_SIGNAL_RUN_CLEANUP | G_SIGNAL_ACTION,
 		              0,
 		              NULL, NULL,
-		              g_cclosure_marshal_VOID__VOID,
-		              G_TYPE_NONE,
+		              brasero_marshal_BOOLEAN__VOID,
+		              G_TYPE_BOOLEAN,
 			      0,
 		              G_TYPE_NONE);
 	session_cfg_signals [IS_VALID_SIGNAL] =
