@@ -43,8 +43,6 @@
 #include <glib/gstdio.h>
 #include <glib/gi18n-lib.h>
 
-#include "brasero-session.h"
-#include "brasero-track.h"
 
 #include "burn-basics.h"
 #include "burn-debug.h"
@@ -55,11 +53,26 @@
 #include "brasero-drive.h"
 #include "brasero-medium-monitor.h"
 
+#include "brasero-tags.h"
+#include "brasero-session.h"
+#include "brasero-track.h"
 #include "brasero-track-disc.h"
 
+
 G_DEFINE_TYPE (BraseroBurnSession, brasero_burn_session, G_TYPE_OBJECT);
 #define BRASERO_BURN_SESSION_PRIVATE(o)  (G_TYPE_INSTANCE_GET_PRIVATE ((o), BRASERO_TYPE_BURN_SESSION, BraseroBurnSessionPrivate))
 
+/**
+ * SECTION:brasero-session
+ * @short_description: Store parameters when burning and blanking
+ * @see_also: #BraseroBurn, #BraseroSessionCfg
+ * @include: brasero-session.h
+ *
+ * This object stores all parameters for all operations performed by #BraseroBurn such as
+ * burning, blanking and checksuming. To have this object configured automatically see
+ * #BraseroSessionCfg.
+ **/
+
 struct _BraseroSessionSetting {
 	BraseroDrive *burner;
 
@@ -108,6 +121,7 @@ typedef struct _BraseroBurnSessionPrivate BraseroBurnSessionPrivate;
 #define BRASERO_STR_EQUAL(a, b)	((!(a) && !(b)) || ((a) && (b) && !strcmp ((a), (b))))
 
 typedef enum {
+	TAG_CHANGED_SIGNAL,
 	FLAGS_CHANGED_SIGNAL,
 	TRACK_ADDED_SIGNAL,
 	TRACK_REMOVED_SIGNAL,
@@ -140,7 +154,7 @@ brasero_session_settings_clean (BraseroSessionSetting *settings)
 	memset (settings, 0, sizeof (BraseroSessionSetting));
 }
 
-void
+static void
 brasero_session_settings_copy (BraseroSessionSetting *dest,
 			       BraseroSessionSetting *original)
 {
@@ -231,6 +245,21 @@ brasero_burn_session_free_tracks (BraseroBurnSession *self)
 	}
 }
 
+
+/**
+ * brasero_burn_session_add_track:
+ * @session: a #BraseroBurnSession
+ * @new_track: a #BraseroTrack or NULL
+ * @sibling: a #BraseroTrack or NULL
+ *
+ * Inserts a new track after @sibling or appended if @sibling is NULL. If @track is NULL then all tracks
+ * already in @session will be removed.
+ * NOTE: if there are already #BraseroTrack objects inserted in the session and if they are not
+ * of the same type as @new_track then they will be removed before the insertion of @new_track.
+ *
+ * Return value: a #BraseroBurnResult. BRASERO_BURN_OK if the track was successfully inserted or BRASERO_BURN_ERR otherwise.
+ **/
+
 BraseroBurnResult
 brasero_burn_session_add_track (BraseroBurnSession *self,
 				BraseroTrack *new_track,
@@ -242,6 +271,12 @@ brasero_burn_session_add_track (BraseroBurnSession *self,
 
 	priv = BRASERO_BURN_SESSION_PRIVATE (self);
 
+	/* Prevent adding the same tracks several times */
+	if (g_slist_find (priv->tracks, new_track)) {
+		BRASERO_BURN_LOG ("Tried to add the same track multiple times");
+		return BRASERO_BURN_OK;
+	}
+
 	if (!new_track) {
 		if (!priv->tracks)
 			return BRASERO_BURN_OK;
@@ -292,6 +327,17 @@ brasero_burn_session_add_track (BraseroBurnSession *self,
 	return BRASERO_BURN_OK;
 }
 
+/**
+ * brasero_burn_session_move_track:
+ * @session: a #BraseroBurnSession
+ * @track: a #BraseroTrack
+ * @sibling: a #BraseroTrack or NULL
+ *
+ * Moves @track after @sibling; if @sibling is NULL then it is appended.
+ *
+ * Return value: a #BraseroBurnResult. BRASERO_BURN_OK if the track was successfully moved or BRASERO_BURN_ERR otherwise.
+ **/
+
 BraseroBurnResult
 brasero_burn_session_move_track (BraseroBurnSession *session,
 				 BraseroTrack *track,
@@ -333,6 +379,16 @@ brasero_burn_session_move_track (BraseroBurnSession *session,
 	return BRASERO_BURN_OK;
 }
 
+/**
+ * brasero_burn_session_remove_track:
+ * @session: a #BraseroBurnSession
+ * @track: a #BraseroTrack
+ *
+ * Removes @track from @session.
+ *
+ * Return value: a #BraseroBurnResult. BRASERO_BURN_OK if the track was successfully removed or BRASERO_BURN_ERR otherwise.
+ **/
+
 BraseroBurnResult
 brasero_burn_session_remove_track (BraseroBurnSession *session,
 				   BraseroTrack *track)
@@ -361,6 +417,15 @@ brasero_burn_session_remove_track (BraseroBurnSession *session,
 	return BRASERO_BURN_OK;
 }
 
+/**
+ * brasero_burn_session_get_tracks:
+ * @session: a #BraseroBurnSession
+ *
+ * Returns the list of #BraseroTrack added to @session.
+ *
+ * Return value: a #GSList or #BraseroTrack object. Do not unref the objects in the list nor destroy the list.
+ **/
+
 GSList *
 brasero_burn_session_get_tracks (BraseroBurnSession *self)
 {
@@ -373,6 +438,19 @@ brasero_burn_session_get_tracks (BraseroBurnSession *self)
 	return priv->tracks;
 }
 
+/**
+ * brasero_burn_session_get_status:
+ * @session: a #BraseroBurnSession
+ * @status: a #BraseroTrackStatus
+ *
+ * Sets @status to reflect whether @session is ready to be used.
+ *
+ * Return value: a #BraseroBurnResult.
+ * BRASERO_BURN_OK if it was successful
+ * BRASERO_BURN_NOT_READY if @track needs more time for processing
+ * BRASERO_BURN_ERR if something is wrong or if it is empty
+ **/
+
 BraseroBurnResult
 brasero_burn_session_get_status (BraseroBurnSession *session,
 				 BraseroStatus *status)
@@ -429,6 +507,20 @@ brasero_burn_session_get_status (BraseroBurnSession *session,
 	return BRASERO_BURN_OK;
 }
 
+/**
+ * brasero_burn_session_get_size:
+ * @session: a #BraseroBurnSession
+ * @blocks: a #goffset or NULL
+ * @bytes: a #goffset or NULL
+ *
+ * Returns the size of the data contained by @session in bytes or in sectors
+ *
+ * Return value: a #BraseroBurnResult.
+ * BRASERO_BURN_OK if it was successful
+ * BRASERO_BURN_NOT_READY if @track needs more time for processing the size
+ * BRASERO_BURN_ERR if something is wrong or if it is empty
+ **/
+
 BraseroBurnResult
 brasero_burn_session_get_size (BraseroBurnSession *session,
 			       goffset *blocks,
@@ -474,6 +566,17 @@ brasero_burn_session_get_size (BraseroBurnSession *session,
 	return BRASERO_BURN_OK;
 }
 
+/**
+ * brasero_burn_session_get_input_type:
+ * @session: a #BraseroBurnSession
+ * @type: a #BraseroTrackType or NULL
+ *
+ * Sets @type to reflect the type of data contained in @session
+ *
+ * Return value: a #BraseroBurnResult.
+ * BRASERO_BURN_OK if it was successful
+ **/
+
 BraseroBurnResult
 brasero_burn_session_get_input_type (BraseroBurnSession *self,
 				     BraseroTrackType *type)
@@ -496,10 +599,6 @@ brasero_burn_session_get_input_type (BraseroBurnSession *self,
 	return BRASERO_BURN_OK;
 }
 
-/**
- *
- */
-
 static void
 brasero_burn_session_dest_media_added (BraseroDrive *drive,
 				       BraseroMedium *medium,
@@ -523,6 +622,15 @@ brasero_burn_session_dest_media_removed (BraseroDrive *drive,
 		       medium);
 }
 
+/**
+ * brasero_burn_session_set_burner:
+ * @session: a #BraseroBurnSession
+ * @drive: a #BraseroDrive
+ *
+ * Sets the #BraseroDrive that should be used to burn the session contents.
+ *
+ **/
+
 void
 brasero_burn_session_set_burner (BraseroBurnSession *self,
 				 BraseroDrive *drive)
@@ -539,7 +647,7 @@ brasero_burn_session_set_burner (BraseroBurnSession *self,
 
 	former = brasero_drive_get_medium (priv->settings->burner);
 	if (former)
-		former = g_object_ref (former);
+		g_object_ref (former);
 
 	/* If there was no drive before no need for a changing signal */
 	if (priv->settings->burner) {
@@ -576,10 +684,20 @@ brasero_burn_session_set_burner (BraseroBurnSession *self,
 		       brasero_burn_session_signals [OUTPUT_CHANGED_SIGNAL],
 		       0,
 		       former);
+
 	if (former)
 		g_object_unref (former);
 }
 
+/**
+ * brasero_burn_session_get_burner:
+ * @session: a #BraseroBurnSession
+ *
+ * Returns the #BraseroDrive that should be used to burn the session contents.
+ *
+ * Return value: a #BraseroDrive or NULL. Do not unref after use.
+ **/
+
 BraseroDrive *
 brasero_burn_session_get_burner (BraseroBurnSession *self)
 {
@@ -591,8 +709,22 @@ brasero_burn_session_get_burner (BraseroBurnSession *self)
 	return priv->settings->burner;
 }
 
+/**
+ * brasero_burn_session_set_rate:
+ * @session: a #BraseroBurnSession
+ * @rate: a #guint64
+ *
+ * Sets the speed at which the medium should be burnt.
+ * NOTE: before using this function a #BraseroDrive should have been
+ * set with brasero_burn_session_set_burner ().
+ *
+ * Return value: a #BraseroBurnResult.
+ * BRASERO_BURN_OK if it was successful; BRASERO_BURN_ERR otherwise.
+ **/
+
 BraseroBurnResult
-brasero_burn_session_set_rate (BraseroBurnSession *self, guint64 rate)
+brasero_burn_session_set_rate (BraseroBurnSession *self,
+                               guint64 rate)
 {
 	BraseroBurnSessionPrivate *priv;
 
@@ -607,6 +739,17 @@ brasero_burn_session_set_rate (BraseroBurnSession *self, guint64 rate)
 	return BRASERO_BURN_OK;
 }
 
+/**
+ * brasero_burn_session_get_rate:
+ * @session: a #BraseroBurnSession
+ *
+ * Returns the speed at which the medium should be burnt.
+ * NOTE: before using this function a #BraseroDrive should have been
+ * set with brasero_burn_session_set_burner ().
+ *
+ * Return value: a #guint64 or 0.
+ **/
+
 guint64
 brasero_burn_session_get_rate (BraseroBurnSession *self)
 {
@@ -630,9 +773,23 @@ brasero_burn_session_get_rate (BraseroBurnSession *self)
 }
 
 /**
- * This function returns a path only if we should output to a file image
- * and not burn.
- */
+ * brasero_burn_session_get_output:
+ * @session: a #BraseroBurnSession
+ * @image: a #gchar or NULL
+ * @toc: a #gchar or NULL
+ *
+ * When the contents of @session should be written to a
+ * file then this function returns the image path (and if
+ * necessary a toc path).
+ * @image and @toc should be freed if not used anymore.
+ *
+ * NOTE: before using this function a #BraseroDrive should have been
+ * set with brasero_burn_session_set_burner () and it should be the
+ * fake drive (see brasero_drive_is_fake ()).
+ *
+ * Return value: a #BraseroBurnResult.
+ * BRASERO_BURN_OK if it was successful; BRASERO_BURN_ERR otherwise.
+ **/
 
 BraseroBurnResult
 brasero_burn_session_get_output (BraseroBurnSession *self,
@@ -668,7 +825,6 @@ brasero_burn_session_get_output_path_real (BraseroBurnSession *self,
 
 	image = g_strdup (priv->settings->image);
 	toc = g_strdup (priv->settings->toc);
-
 	if (!image && !toc)
 		return BRASERO_BURN_ERR;
 
@@ -707,6 +863,20 @@ brasero_burn_session_get_output_path_real (BraseroBurnSession *self,
 	return BRASERO_BURN_OK;
 }
 
+/**
+ * brasero_burn_session_get_output_format:
+ * @session: a #BraseroBurnSession
+ *
+ * When the contents of @session should be written to a
+ * file then this function returns the format of the image to write.
+ *
+ * NOTE: before using this function a #BraseroDrive should have been
+ * set with brasero_burn_session_set_burner () and it should be the
+ * fake drive (see brasero_drive_is_fake ()).
+ *
+ * Return value: a #BraseroImageFormat. The format of the image to be written.
+ **/
+
 BraseroImageFormat
 brasero_burn_session_get_output_format (BraseroBurnSession *self)
 {
@@ -805,7 +975,7 @@ brasero_burn_session_set_output_image_real (BraseroBurnSession *self,
 		brasero_burn_session_set_image_output_real (self, format, image, toc);
 
 		monitor = brasero_medium_monitor_get_default ();
-		list = brasero_medium_monitor_get_media (monitor,BRASERO_MEDIA_TYPE_FILE);
+		list = brasero_medium_monitor_get_media (monitor, BRASERO_MEDIA_TYPE_FILE);
 		drive = brasero_medium_get_drive (list->data);
 		brasero_burn_session_set_burner (self, drive);
 		g_object_unref (monitor);
@@ -822,6 +992,65 @@ brasero_burn_session_set_output_image_real (BraseroBurnSession *self,
 	return BRASERO_BURN_OK;
 }
 
+/**
+ * brasero_burn_session_set_image_output_format:
+ * @session: a #BraseroBurnSession
+ * @format: a #BraseroImageFormat
+ *
+ * When the contents of @session should be written to a
+ * file, this function sets format of the image that will be
+ * created.
+ *
+ * NOTE: after a call to this function the #BraseroDrive for
+ * @session will be the fake #BraseroDrive.
+ *
+ * Return value: a #BraseroBurnResult. BRASERO_BURN_OK if it was successfully set;
+ * BRASERO_BURN_ERR otherwise.
+ **/
+
+BraseroBurnResult
+brasero_burn_session_set_image_output_format (BraseroBurnSession *self,
+					    BraseroImageFormat format)
+{
+	BraseroBurnSessionClass *klass;
+	BraseroBurnSessionPrivate *priv;
+	BraseroBurnResult res;
+	gchar *image;
+	gchar *toc;
+
+	g_return_val_if_fail (BRASERO_IS_BURN_SESSION (self), BRASERO_BURN_ERR);
+
+	priv = BRASERO_BURN_SESSION_PRIVATE (self);
+	klass = BRASERO_BURN_SESSION_GET_CLASS (self);
+
+	image = g_strdup (priv->settings->image);
+	toc = g_strdup (priv->settings->toc);
+	res = klass->set_output_image (self, format, image, toc);
+	g_free (image);
+	g_free (toc);
+
+	return res;
+}
+
+/**
+ * brasero_burn_session_set_image_output_full:
+ * @session: a #BraseroBurnSession
+ * @format: a #BraseroImageFormat
+ * @image: a #gchar or NULL
+ * @toc: a #gchar or NULL
+ *
+ * When the contents of @session should be written to a
+ * file, this function sets the different parameters of this
+ * image like its path (and the one of the associated toc if
+ * necessary) and its format.
+ *
+ * NOTE: after a call to this function the #BraseroDrive for
+ * @session will be the fake #BraseroDrive.
+ *
+ * Return value: a #BraseroBurnResult. BRASERO_BURN_OK if it was successfully set;
+ * BRASERO_BURN_ERR otherwise.
+ **/
+
 BraseroBurnResult
 brasero_burn_session_set_image_output_full (BraseroBurnSession *self,
 					    BraseroImageFormat format,
@@ -837,8 +1066,16 @@ brasero_burn_session_set_image_output_full (BraseroBurnSession *self,
 }
 
 /**
+ * brasero_burn_session_set_tmpdir:
+ * @session: a #BraseroBurnSession
+ * @path: a #gchar or NULL
  *
- */
+ * Sets the path of the directory in which to write temporary directories and files.
+ * If set to NULL then the result of g_get_tmp_dir () will be used.
+ *
+ * Return value: a #BraseroBurnResult. BRASERO_BURN_OK if it was successfully set;
+ * BRASERO_BURN_ERR otherwise.
+ **/
 
 BraseroBurnResult
 brasero_burn_session_set_tmpdir (BraseroBurnSession *self,
@@ -865,6 +1102,15 @@ brasero_burn_session_set_tmpdir (BraseroBurnSession *self,
 	return BRASERO_BURN_OK;
 }
 
+/**
+ * brasero_burn_session_get_tmpdir:
+ * @session: a #BraseroBurnSession
+ *
+ * Returns the path of the directory in which to write temporary directories and files.
+ *
+ * Return value: a #gchar. The path to the temporary directory.
+ **/
+
 const gchar *
 brasero_burn_session_get_tmpdir (BraseroBurnSession *self)
 {
@@ -876,6 +1122,21 @@ brasero_burn_session_get_tmpdir (BraseroBurnSession *self)
 	return priv->settings->tmpdir? priv->settings->tmpdir:g_get_tmp_dir ();
 }
 
+/**
+ * brasero_burn_session_get_tmp_dir:
+ * @session: a #BraseroBurnSession
+ * @path: a #gchar or NULL
+ * @error: a #GError
+ *
+ * Creates then returns (in @path) a temporary directory at the proper location.
+ * On error, @error is set appropriately.
+ * See brasero_burn_session_set_tmpdir ().
+ * This function is used internally and is not public API.
+ *
+ * Return value: a #BraseroBurnResult. BRASERO_BURN_OK if it was successfully set;
+ * BRASERO_BURN_ERR otherwise.
+ **/
+
 BraseroBurnResult
 brasero_burn_session_get_tmp_dir (BraseroBurnSession *self,
 				  gchar **path,
@@ -925,6 +1186,23 @@ brasero_burn_session_get_tmp_dir (BraseroBurnSession *self,
 	return BRASERO_BURN_OK;
 }
 
+/**
+ * brasero_burn_session_get_tmp_file:
+ * @session: a #BraseroBurnSession
+ * @suffix: a #gchar
+ * @path: a #gchar or NULL
+ * @error: a #GError
+ *
+ * Creates then returns (in @path) a temporary file at the proper location. Its name
+ * will be appended with suffix.
+ * On error, @error is set appropriately.
+ * See brasero_burn_session_set_tmpdir ().
+ * This function is used internally and is not public API.
+ *
+ * Return value: a #BraseroBurnResult. BRASERO_BURN_OK if it was successfully set;
+ * BRASERO_BURN_ERR otherwise.
+ **/
+
 BraseroBurnResult
 brasero_burn_session_get_tmp_file (BraseroBurnSession *self,
 				   const gchar *suffix,
@@ -1021,6 +1299,10 @@ brasero_burn_session_get_image_complement (BraseroBurnSession *self,
 	return retval;
 }
 
+/**
+ * This function is used internally and is not public API
+ */
+
 BraseroBurnResult
 brasero_burn_session_get_tmp_image (BraseroBurnSession *self,
 				    BraseroImageFormat format,
@@ -1078,6 +1360,15 @@ brasero_burn_session_get_tmp_image (BraseroBurnSession *self,
  * used to modify session flags.
  */
 
+/**
+ * brasero_burn_session_set_flags:
+ * @session: a #BraseroBurnSession
+ * @flags: a #BraseroBurnFlag
+ *
+ * Replaces the current flags set in @session with @flags.
+ *
+ **/
+
 void
 brasero_burn_session_set_flags (BraseroBurnSession *self,
 			        BraseroBurnFlag flags)
@@ -1096,6 +1387,15 @@ brasero_burn_session_set_flags (BraseroBurnSession *self,
 		       0);
 }
 
+/**
+ * brasero_burn_session_add_flag:
+ * @session: a #BraseroBurnSession
+ * @flags: a #BraseroBurnFlag
+ *
+ * Merges the current flags set in @session with @flags.
+ *
+ **/
+
 void
 brasero_burn_session_add_flag (BraseroBurnSession *self,
 			       BraseroBurnFlag flag)
@@ -1114,6 +1414,15 @@ brasero_burn_session_add_flag (BraseroBurnSession *self,
 		       0);
 }
 
+/**
+ * brasero_burn_session_remove_flag:
+ * @session: a #BraseroBurnSession
+ * @flags: a #BraseroBurnFlag
+ *
+ * Removes @flags from the current flags set for @session.
+ *
+ **/
+
 void
 brasero_burn_session_remove_flag (BraseroBurnSession *self,
 				  BraseroBurnFlag flag)
@@ -1132,6 +1441,15 @@ brasero_burn_session_remove_flag (BraseroBurnSession *self,
 		       0);
 }
 
+/**
+ * brasero_burn_session_get_flags:
+ * @session: a #BraseroBurnSession
+ *
+ * Returns the current flags set for @session.
+ *
+ * Return value: a #BraseroBurnFlag.
+ **/
+
 BraseroBurnFlag
 brasero_burn_session_get_flags (BraseroBurnSession *self)
 {
@@ -1146,7 +1464,16 @@ brasero_burn_session_get_flags (BraseroBurnSession *self)
 /**
  * Used to set the label or the title of an album. 
  */
- 
+
+/**
+ * brasero_burn_session_set_label:
+ * @session: a #BraseroBurnSession
+ * @label: a #gchar
+ *
+ * Sets the label for @session.
+ *
+ **/
+
 void
 brasero_burn_session_set_label (BraseroBurnSession *self,
 				const gchar *label)
@@ -1184,6 +1511,15 @@ brasero_burn_session_set_label (BraseroBurnSession *self,
 	}
 }
 
+/**
+ * brasero_burn_session_get_label:
+ * @session: a #BraseroBurnSession
+ *
+ * Returns the label (a string) set for @session.
+ *
+ * Return value: a #gchar or NULL. Do not free after use.
+ **/
+
 const gchar *
 brasero_burn_session_get_label (BraseroBurnSession *self)
 {
@@ -1204,6 +1540,19 @@ brasero_burn_session_tag_value_free (gpointer user_data)
 	g_free (value);
 }
 
+/**
+ * brasero_burn_session_tag_remove:
+ * @session: a #BraseroBurnSession
+ * @tag: a #gchar *
+ *
+ * Removes a value associated with @session through
+ * brasero_session_tag_add ().
+ *
+ * Return value: a #BraseroBurnResult.
+ * BRASERO_BURN_OK if the retrieval was successful
+ * BRASERO_BURN_ERR otherwise
+ **/
+
 BraseroBurnResult
 brasero_burn_session_tag_remove (BraseroBurnSession *self,
 				 const gchar *tag)
@@ -1211,15 +1560,38 @@ brasero_burn_session_tag_remove (BraseroBurnSession *self,
 	BraseroBurnSessionPrivate *priv;
 
 	g_return_val_if_fail (BRASERO_IS_BURN_SESSION (self), BRASERO_BURN_ERR);
+	g_return_val_if_fail (tag != NULL, BRASERO_BURN_ERR);
 
 	priv = BRASERO_BURN_SESSION_PRIVATE (self);
 	if (!priv->tags)
 		return BRASERO_BURN_ERR;
 
 	g_hash_table_remove (priv->tags, tag);
+
+	g_signal_emit (self,
+	               brasero_burn_session_signals [TAG_CHANGED_SIGNAL],
+	               0,
+	               tag);
 	return BRASERO_BURN_OK;
 }
 
+/**
+ * brasero_burn_session_tag_add:
+ * @session: a #BraseroBurnSession
+ * @tag: a #gchar *
+ * @value: a #GValue *
+ *
+ * Associates a new @tag with @session. This can be used
+ * to pass arbitrary information for plugins, like parameters
+ * for video discs, ...
+ * NOTE: the #BraseroBurnSession object takes ownership of @value.
+ * See brasero-tags.h for a list of knowns tags.
+ *
+ * Return value: a #BraseroBurnResult.
+ * BRASERO_BURN_OK if it was successful,
+ * BRASERO_BURN_ERR otherwise.
+ **/
+
 BraseroBurnResult
 brasero_burn_session_tag_add (BraseroBurnSession *self,
 			      const gchar *tag,
@@ -1228,6 +1600,7 @@ brasero_burn_session_tag_add (BraseroBurnSession *self,
 	BraseroBurnSessionPrivate *priv;
 
 	g_return_val_if_fail (BRASERO_IS_BURN_SESSION (self), BRASERO_BURN_ERR);
+	g_return_val_if_fail (tag != NULL, BRASERO_BURN_ERR);
 
 	priv = BRASERO_BURN_SESSION_PRIVATE (self);
 	if (!priv->tags)
@@ -1236,9 +1609,62 @@ brasero_burn_session_tag_add (BraseroBurnSession *self,
 						    g_free,
 						    brasero_burn_session_tag_value_free);
 	g_hash_table_insert (priv->tags, g_strdup (tag), value);
+	g_signal_emit (self,
+	               brasero_burn_session_signals [TAG_CHANGED_SIGNAL],
+	               0,
+	               tag);
+
 	return BRASERO_BURN_OK;
 }
 
+/**
+ * brasero_burn_session_tag_add_int:
+ * @session: a #BraseroBurnSession
+ * @tag: a #gchar *
+ * @value: a #gint
+ *
+ * Associates a new @tag with @session. This can be used
+ * to pass arbitrary information for plugins, like parameters
+ * for video discs, ...
+ * See brasero-tags.h for a list of knowns tags.
+ *
+ * Return value: a #BraseroBurnResult.
+ * BRASERO_BURN_OK if it was successful,
+ * BRASERO_BURN_ERR otherwise.
+ **/
+
+BraseroBurnResult
+brasero_burn_session_tag_add_int (BraseroBurnSession *self,
+                                  const gchar *tag,
+                                  gint value)
+{
+	GValue *gvalue;
+
+	g_return_val_if_fail (BRASERO_IS_BURN_SESSION (self), BRASERO_BURN_ERR);
+	g_return_val_if_fail (tag != NULL, BRASERO_BURN_ERR);
+
+	gvalue = g_new0 (GValue, 1);
+	g_value_init (gvalue, G_TYPE_INT);
+	g_value_set_int (gvalue, value);
+
+	return brasero_burn_session_tag_add (self, tag, gvalue);
+}
+
+/**
+ * brasero_burn_session_tag_lookup:
+ * @session: a #BraseroBurnSession
+ * @tag: a #gchar *
+ * @value: a #GValue
+ *
+ * Retrieves a value associated with @session through
+ * brasero_session_tag_add () and stores it in @value. Do
+ * not destroy @value afterwards as it is not a copy.
+ *
+ * Return value: a #BraseroBurnResult.
+ * BRASERO_BURN_OK if the retrieval was successful
+ * BRASERO_BURN_ERR otherwise
+ **/
+
 BraseroBurnResult
 brasero_burn_session_tag_lookup (BraseroBurnSession *self,
 				 const gchar *tag,
@@ -1248,6 +1674,7 @@ brasero_burn_session_tag_lookup (BraseroBurnSession *self,
 	gpointer data;
 
 	g_return_val_if_fail (BRASERO_IS_BURN_SESSION (self), BRASERO_BURN_ERR);
+	g_return_val_if_fail (tag != NULL, BRASERO_BURN_ERR);
 
 	priv = BRASERO_BURN_SESSION_PRIVATE (self);
 	if (!value)
@@ -1260,10 +1687,37 @@ brasero_burn_session_tag_lookup (BraseroBurnSession *self,
 	if (!data)
 		return BRASERO_BURN_ERR;
 
-	*value = data;
+	/* value can be NULL if the function was just called
+	 * to check whether a tag was set. */
+	if (value)
+		*value = data;
 	return BRASERO_BURN_OK;
 }
 
+/**
+ * brasero_burn_session_tag_lookup_int:
+ * @session: a #BraseroBurnSession
+ * @tag: a #gchar
+ *
+ * Retrieves an int value associated with @session through
+ * brasero_session_tag_add () and returns it.
+ *
+ * Return value: a #gint.
+ **/
+
+gint
+brasero_burn_session_tag_lookup_int (BraseroBurnSession *self,
+                                     const gchar *tag)
+{
+	GValue *value = NULL;
+
+	brasero_burn_session_tag_lookup (self, tag, &value);
+	if (!value || !G_VALUE_HOLDS_INT (value))
+		return 0;
+
+	return g_value_get_int (value);
+}
+
 /**
  * Used to save and restore settings/sources
  */
@@ -1321,7 +1775,6 @@ brasero_burn_session_pop_settings (BraseroBurnSession *self)
 	brasero_session_settings_copy (priv->settings, settings);
 
 	brasero_session_settings_free (settings);
-
 	if (priv->settings->burner) {
 		priv->dest_added_sig = g_signal_connect (priv->settings->burner,
 							 "medium-added",
@@ -1337,6 +1790,7 @@ brasero_burn_session_pop_settings (BraseroBurnSession *self)
 		       brasero_burn_session_signals [OUTPUT_CHANGED_SIGNAL],
 		       0,
 		       former);
+
 	if (former)
 		g_object_unref (former);
 }
@@ -1404,10 +1858,6 @@ brasero_burn_session_pop_tracks (BraseroBurnSession *self)
 	return BRASERO_BURN_RETRY;
 }
 
-/**
- *
- */
-
 gboolean
 brasero_burn_session_is_dest_file (BraseroBurnSession *self)
 {
@@ -1437,6 +1887,47 @@ brasero_burn_session_get_dest_media (BraseroBurnSession *self)
 	return brasero_medium_get_status (medium);
 }
 
+/**
+ * brasero_burn_session_get_available_medium_space:
+ * @session: a #BraseroBurnSession
+ *
+ * Returns the maximum space available for the
+ * medium currently inserted in the #BraseroDrive
+ * set as burner with brasero_burn_session_set_burner ().
+ * This takes into account flags.
+ *
+ * Return value: a #goffset.
+ **/
+
+goffset
+brasero_burn_session_get_available_medium_space (BraseroBurnSession *session)
+{
+	BraseroDrive *burner;
+	BraseroBurnFlag flags;
+	BraseroMedium *medium;
+	goffset available_blocks = 0;
+
+	/* Retrieve the size available for burning */
+	burner = brasero_burn_session_get_burner (session);
+	if (!burner)
+		return 0;
+
+	medium = brasero_drive_get_medium (burner);
+	if (!medium)
+		return 0;
+
+	flags = brasero_burn_session_get_flags (session);
+	if (flags & (BRASERO_BURN_FLAG_MERGE|BRASERO_BURN_FLAG_APPEND))
+		brasero_medium_get_free_space (medium, NULL, &available_blocks);
+	else if (brasero_burn_session_can_blank (session) == BRASERO_BURN_OK)
+		brasero_medium_get_capacity (medium, NULL, &available_blocks);
+	else
+		brasero_medium_get_free_space (medium, NULL, &available_blocks);
+
+	BRASERO_BURN_LOG ("Available space on medium %" G_GINT64_FORMAT, available_blocks);
+	return available_blocks;
+}
+
 BraseroMedium *
 brasero_burn_session_get_src_medium (BraseroBurnSession *self)
 {
@@ -1651,6 +2142,42 @@ brasero_burn_session_start (BraseroBurnSession *self)
 	BRASERO_BURN_LOG_TYPE (type, "Input\t=");
 	BRASERO_BURN_LOG_FLAGS (priv->settings->flags, "flags\t=");
 
+	/* also try all known tags */
+	if (brasero_track_type_get_has_stream (type)
+	&&  BRASERO_STREAM_FORMAT_HAS_VIDEO (brasero_track_type_get_stream_format (type))) {
+		GValue *value;
+
+		BRASERO_BURN_LOG ("Tags set:");
+
+		value = NULL;
+		brasero_burn_session_tag_lookup (self,
+						 BRASERO_DVD_STREAM_FORMAT,
+						 &value);
+		if (value)
+			BRASERO_BURN_LOG ("Stream format %i", g_value_get_int (value));
+
+		value = NULL;
+		brasero_burn_session_tag_lookup (self,
+						 BRASERO_VCD_TYPE,
+						 &value);
+		if (value)
+			BRASERO_BURN_LOG ("(S)VCD type %i", g_value_get_int (value));
+
+		value = NULL;
+		brasero_burn_session_tag_lookup (self,
+						 BRASERO_VIDEO_OUTPUT_FRAMERATE,
+						 &value);
+		if (value)
+			BRASERO_BURN_LOG ("Framerate %i", g_value_get_int (value));
+
+		value = NULL;
+		brasero_burn_session_tag_lookup (self,
+						 BRASERO_VIDEO_OUTPUT_ASPECT,
+						 &value);
+		if (value)
+			BRASERO_BURN_LOG ("Aspect %i", g_value_get_int (value));
+	}
+
 	if (!brasero_burn_session_is_dest_file (self)) {
 		BraseroMedium *medium;
 
@@ -1855,7 +2382,14 @@ brasero_burn_session_class_init (BraseroBurnSessionClass *klass)
 	klass->get_output_format = brasero_burn_session_get_output_format_real;
 	klass->set_output_image = brasero_burn_session_set_output_image_real;
 
-	/* This is to delay the setting of track source until we know all settings */
+	/**
+ 	* BraseroBurnSession::output-changed:
+ 	* @session: the object which received the signal
+  	* @former_medium: the medium which was previously set
+	*
+ 	* This signal gets emitted when the medium to burn to changed.
+ 	*
+ 	*/
 	brasero_burn_session_signals [OUTPUT_CHANGED_SIGNAL] =
 	    g_signal_new ("output_changed",
 			  BRASERO_TYPE_BURN_SESSION,
@@ -1867,6 +2401,15 @@ brasero_burn_session_class_init (BraseroBurnSessionClass *klass)
 			  G_TYPE_NONE,
 			  1,
 			  BRASERO_TYPE_MEDIUM);
+
+	/**
+ 	* BraseroBurnSession::track-added:
+ 	* @session: the object which received the signal
+  	* @track: the track that was added
+	*
+ 	* This signal gets emitted when a track is added to @session.
+ 	*
+ 	*/
 	brasero_burn_session_signals [TRACK_ADDED_SIGNAL] =
 	    g_signal_new ("track_added",
 			  BRASERO_TYPE_BURN_SESSION,
@@ -1878,6 +2421,16 @@ brasero_burn_session_class_init (BraseroBurnSessionClass *klass)
 			  G_TYPE_NONE,
 			  1,
 			  BRASERO_TYPE_TRACK);
+
+	/**
+ 	* BraseroBurnSession::track-removed:
+ 	* @session: the object which received the signal
+  	* @track: the track that was removed
+	* @former_position: the former position of @track
+	*
+ 	* This signal gets emitted when a track is removed from @session.
+ 	*
+ 	*/
 	brasero_burn_session_signals [TRACK_REMOVED_SIGNAL] =
 	    g_signal_new ("track_removed",
 			  BRASERO_TYPE_BURN_SESSION,
@@ -1890,6 +2443,15 @@ brasero_burn_session_class_init (BraseroBurnSessionClass *klass)
 			  2,
 			  BRASERO_TYPE_TRACK,
 			  G_TYPE_UINT);
+
+	/**
+ 	* BraseroBurnSession::track-changed:
+ 	* @session: the object which received the signal
+  	* @track: the track that changed
+	*
+ 	* This signal gets emitted when the contents of a track changed.
+ 	*
+ 	*/
 	brasero_burn_session_signals [TRACK_CHANGED_SIGNAL] =
 	    g_signal_new ("track_changed",
 			  BRASERO_TYPE_BURN_SESSION,
@@ -1901,6 +2463,14 @@ brasero_burn_session_class_init (BraseroBurnSessionClass *klass)
 			  G_TYPE_NONE,
 			  1,
 			  BRASERO_TYPE_TRACK);
+
+	/**
+ 	* BraseroBurnSession::flags-changed:
+ 	* @session: the object which received the signal
+	*
+ 	* This signal gets emitted when the flags changed for @session.
+ 	*
+ 	*/
 	brasero_burn_session_signals [FLAGS_CHANGED_SIGNAL] =
 	    g_signal_new ("flags_changed",
 			  BRASERO_TYPE_BURN_SESSION,
@@ -1911,8 +2481,36 @@ brasero_burn_session_class_init (BraseroBurnSessionClass *klass)
 			  g_cclosure_marshal_VOID__VOID,
 			  G_TYPE_NONE,
 			  0);
+
+	/**
+ 	* BraseroBurnSession::tag-changed:
+ 	* @session: the object which received the signal
+	*
+ 	* This signal gets emitted when a tag changed for @session (whether it
+	* was removed, added, or it changed).
+ 	*
+ 	*/
+	brasero_burn_session_signals [TAG_CHANGED_SIGNAL] =
+	    g_signal_new ("tag_changed",
+			  BRASERO_TYPE_BURN_SESSION,
+			  G_SIGNAL_RUN_FIRST,
+			  0,
+			  NULL,
+			  NULL,
+			  g_cclosure_marshal_VOID__STRING,
+			  G_TYPE_NONE,
+			  1,
+	                  G_TYPE_STRING);
 }
 
+/**
+ * brasero_burn_session_new:
+ *
+ * Returns a new #BraseroBurnSession object.
+ *
+ * Return value: a #BraseroBurnSession.
+ **/
+
 BraseroBurnSession *
 brasero_burn_session_new ()
 {
