@@ -1468,6 +1468,26 @@ brasero_job_get_action (BraseroJob *self, BraseroJobAction *action)
 	return BRASERO_BURN_OK;
 }
 
+BraseroBurnResult
+brasero_job_get_bus_target_lun (BraseroJob *self, gchar **BTL)
+{
+	BraseroBurnSession *session;
+	BraseroJobPrivate *priv;
+	BraseroDrive *drive;
+
+	BRASERO_JOB_DEBUG (self);
+
+	g_return_val_if_fail (BTL != NULL, BRASERO_BURN_ERR);
+
+	priv = BRASERO_JOB_PRIVATE (self);
+	session = brasero_task_ctx_get_session (priv->ctx);
+
+	drive = brasero_burn_session_get_burner (session);
+	*BTL = brasero_drive_get_bus_target_lun_string (drive);
+
+	return BRASERO_BURN_OK;
+}
+
 BraseroBurnResult
 brasero_job_get_device (BraseroJob *self, gchar **device)
 {
@@ -1785,6 +1805,18 @@ brasero_job_start_progress (BraseroJob *self,
 	return brasero_task_ctx_start_progress (priv->ctx, force);
 }
 
+BraseroBurnResult
+brasero_job_reset_progress (BraseroJob *self)
+{
+	BraseroJobPrivate *priv;
+
+	priv = BRASERO_JOB_PRIVATE (self);
+	if (priv->next)
+		return BRASERO_BURN_ERR;
+
+	return brasero_task_ctx_reset_progress (priv->ctx);
+}
+
 /**
  * these should be used to set the different values of the task by the jobs
  */
