@@ -65,7 +65,7 @@ const gchar *types [] = {	N_("File"),
 				N_("CDROM"),
 				N_("CD-R"),
 				N_("CD-RW"),
-				N_("DVDROM"),
+				N_("DVD-ROM"),
 				N_("DVD-R"),
 				N_("DVD-RW"),
 				N_("DVD+R"),
@@ -114,6 +114,8 @@ struct _BraseroMediumPrivate
 	guint dummy_sao:1;
 	guint dummy_tao:1;
 	guint burnfree:1;
+	guint sao:1;
+	guint tao:1;
 
 	guint blank_command:1;
 	guint write_command:1;
@@ -833,7 +835,7 @@ brasero_medium_get_capacity (BraseroMedium *medium,
  */
 
 static gboolean
-brasero_medium_test_simulate_CD_TAO (BraseroMedium *self,
+brasero_medium_test_CD_TAO_simulate (BraseroMedium *self,
 				     BraseroDeviceHandle *handle,
 				     BraseroScsiErrCode *code)
 {
@@ -859,8 +861,8 @@ brasero_medium_test_simulate_CD_TAO (BraseroMedium *self,
 	}
 
 	desc = hdr->desc;
-	if (!desc->current)
-		BRASERO_MEDIA_LOG ("Feature is not current");
+	priv->tao = (desc->current != 0);
+	BRASERO_MEDIA_LOG ("TAO feature is %s", priv->tao? "supported":"not supported");
 
 	tao_desc = (BraseroScsiCDTAODesc *) desc->data;
 	priv->dummy_tao = tao_desc->dummy != 0;
@@ -876,7 +878,7 @@ brasero_medium_test_simulate_CD_TAO (BraseroMedium *self,
 }
 
 static gboolean
-brasero_medium_test_simulate_CD_SAO (BraseroMedium *self,
+brasero_medium_test_CD_SAO_simulate (BraseroMedium *self,
 				     BraseroDeviceHandle *handle,
 				     BraseroScsiErrCode *code)
 {
@@ -901,8 +903,8 @@ brasero_medium_test_simulate_CD_SAO (BraseroMedium *self,
 	}
 
 	desc = hdr->desc;
-	if (!desc->current)
-		BRASERO_MEDIA_LOG ("Feature is not current");
+	priv->sao = (desc->current != 0);
+	BRASERO_MEDIA_LOG ("SAO feature is %s", priv->sao? "supported":"not supported");
 
 	sao_desc = (BraseroScsiCDSAODesc *) desc->data;
 	priv->dummy_sao = sao_desc->dummy != 0;
@@ -913,9 +915,9 @@ brasero_medium_test_simulate_CD_SAO (BraseroMedium *self,
 }
 
 static gboolean
-brasero_medium_test_simulate_DVDRW (BraseroMedium *self,
-				    BraseroDeviceHandle *handle,
-				    BraseroScsiErrCode *code)
+brasero_medium_test_DVDRW_incremental_simulate (BraseroMedium *self,
+                                                BraseroDeviceHandle *handle,
+                                                BraseroScsiErrCode *code)
 {
 	BraseroScsiDVDRWlessWrtDesc *less_wrt_desc;
 	BraseroScsiGetConfigHdr *hdr = NULL;
@@ -926,8 +928,26 @@ brasero_medium_test_simulate_DVDRW (BraseroMedium *self,
 
 	priv = BRASERO_MEDIUM_PRIVATE (self);
 
+	/* Try incremental feature */
+	BRASERO_MEDIA_LOG ("Checking incremental and simulate feature");
+	result = brasero_mmc2_get_configuration_feature (handle,
+							 BRASERO_SCSI_FEAT_WRT_INCREMENT,
+							 &hdr,
+							 &size,
+							 code);
+	if (result != BRASERO_SCSI_OK) {
+		BRASERO_MEDIA_LOG ("GET CONFIGURATION failed");
+		return FALSE;
+	}
+
+	priv->tao = (hdr->desc->current != 0);
+	g_free (hdr);
+	hdr = NULL;
+
+	BRASERO_MEDIA_LOG ("Incremental feature is %s", priv->tao? "supported":"not supported");
+
 	/* Only DVD-R(W) support simulation */
-	BRASERO_MEDIA_LOG ("Checking simulate (DVD-R/W)");
+	BRASERO_MEDIA_LOG ("Checking (DVD-R(W) simulate)");
 	result = brasero_mmc2_get_configuration_feature (handle,
 							 BRASERO_SCSI_FEAT_WRT_DVD_LESS,
 							 &hdr,
@@ -939,8 +959,11 @@ brasero_medium_test_simulate_DVDRW (BraseroMedium *self,
 	}
 
 	desc = hdr->desc;
-	if (!desc->current)
-		BRASERO_MEDIA_LOG ("Feature is not current");
+
+	/* NOTE: SAO feature is always supported if this feature is current
+	 * See MMC5 5.3.25 Write feature parameters */
+	priv->sao = (desc->current != 0);
+	BRASERO_MEDIA_LOG ("SAO feature is %s", priv->sao? "supported":"not supported");
 
 	less_wrt_desc = (BraseroScsiDVDRWlessWrtDesc *) desc->data;
 	priv->dummy_sao = less_wrt_desc->dummy != 0;
@@ -961,7 +984,7 @@ brasero_medium_test_simulate_DVDRW (BraseroMedium *self,
  */
 
 static void
-brasero_medium_test_simulate_2A (BraseroMedium *self,
+brasero_medium_test_2A_simulate (BraseroMedium *self,
 				 BraseroDeviceHandle *handle,
 				 BraseroScsiErrCode *code)
 {
@@ -973,6 +996,7 @@ brasero_medium_test_simulate_2A (BraseroMedium *self,
 
 	priv = BRASERO_MEDIUM_PRIVATE (self);
 
+	/* FIXME: we need to get a way to get the write types */
 	result = brasero_spc1_mode_sense_get_page (handle,
 						   BRASERO_SPC_PAGE_STATUS,
 						   &data,
@@ -1008,18 +1032,18 @@ brasero_medium_init_caps (BraseroMedium *self,
 
 	priv = BRASERO_MEDIUM_PRIVATE (self);
 
-	/* These special media don't support/need burnfree and simulation */
+	/* These special media don't support/need burnfree, simulation, tao/sao */
 	if (priv->info & (BRASERO_MEDIUM_PLUS|BRASERO_MEDIUM_BD))
 		return;
 
 	if (priv->info & BRASERO_MEDIUM_CD) {
 		/* we have to do both */
-		res = brasero_medium_test_simulate_CD_SAO (self, handle, code);
+		res = brasero_medium_test_CD_SAO_simulate (self, handle, code);
 		if (res)
-			brasero_medium_test_simulate_CD_TAO (self, handle, code);
+			brasero_medium_test_CD_TAO_simulate (self, handle, code);
 	}
 	else
-		res = brasero_medium_test_simulate_DVDRW (self, handle, code);
+		res = brasero_medium_test_DVDRW_incremental_simulate (self, handle, code);
 
 	BRASERO_MEDIA_LOG ("Tested simulation %d %d, burnfree %d",
 			  priv->dummy_tao,
@@ -1031,7 +1055,7 @@ brasero_medium_init_caps (BraseroMedium *self,
 
 	/* it didn't work out as expected use fallback */
 	BRASERO_MEDIA_LOG ("Using fallback 2A page for testing simulation and burnfree");
-	brasero_medium_test_simulate_2A (self, handle, code);
+	brasero_medium_test_2A_simulate (self, handle, code);
 
 	BRASERO_MEDIA_LOG ("Re-tested simulation %d %d, burnfree %d",
 			  priv->dummy_tao,
@@ -1124,7 +1148,9 @@ brasero_medium_get_capacity_DVD_RW (BraseroMedium *self,
 	current = hdr->max_caps;
 	if (!(current->type & BRASERO_SCSI_DESC_FORMATTED)) {
 		BRASERO_MEDIA_LOG ("Unformatted media");
-		priv->info |= BRASERO_MEDIUM_UNFORMATTED;
+		/* If it's sequential, it's not unformatted */
+		if (!(priv->info & BRASERO_MEDIUM_SEQUENTIAL))
+			priv->info |= BRASERO_MEDIUM_UNFORMATTED;
 
 		/* if unformatted, a DVD-RAM will return its maximum formattable
 		 * size in this descriptor and that's what we're looking for. */
@@ -1674,7 +1700,7 @@ brasero_medium_track_set_leadout_DVDR_blank (BraseroMedium *self,
 	 * they are blank. */
 	current = hdr->max_caps;
 	if (current->type & BRASERO_SCSI_DESC_FORMATTED) {
-		BRASERO_MEDIA_LOG ("Unformatted medium");
+		BRASERO_MEDIA_LOG ("Formatted medium");
 		g_free (hdr);
 		return FALSE;
 	}
@@ -2923,22 +2949,21 @@ static gpointer
 brasero_medium_probe_thread (gpointer self)
 {
 	gint counter = 0;
-	const gchar *path;
+	const gchar *device;
 	BraseroScsiErrCode code;
 	BraseroMediumPrivate *priv;
 	BraseroDeviceHandle *handle;
 
 	priv = BRASERO_MEDIUM_PRIVATE (self);
-	path = brasero_drive_get_device (priv->drive);
-	if (!path)
-		path = brasero_drive_get_block_device (priv->drive);
+
 	priv->info = BRASERO_MEDIUM_BUSY;
 
 	/* the drive might be busy (a burning is going on) so we don't block
 	 * but we re-try to open it every second */
-	BRASERO_MEDIA_LOG ("Trying to open device %s", path);
+	device = brasero_drive_get_device (priv->drive);
+	BRASERO_MEDIA_LOG ("Trying to open device %s", device);
 
-	handle = brasero_device_handle_open (path, FALSE, &code);
+	handle = brasero_device_handle_open (device, FALSE, &code);
 	while (!handle && counter <= BRASERO_MEDIUM_OPEN_ATTEMPTS) {
 		sleep (1);
 
@@ -2948,7 +2973,7 @@ brasero_medium_probe_thread (gpointer self)
 		}
 
 		counter ++;
-		handle = brasero_device_handle_open (path, FALSE, &code);
+		handle = brasero_device_handle_open (device, FALSE, &code);
 	}
 
 	if (priv->probe_cancelled) {
@@ -2961,8 +2986,15 @@ brasero_medium_probe_thread (gpointer self)
 
 		/* NOTE: if we wanted to know the status we'd need to read the 
 		 * error code variable which is currently NULL */
-		while (brasero_spc1_test_unit_ready (handle, NULL) != BRASERO_SCSI_OK) {
-			sleep (1);
+		while (brasero_spc1_test_unit_ready (handle, &code) != BRASERO_SCSI_OK) {
+			if (code != BRASERO_SCSI_NOT_READY) {
+				priv->probe = NULL;
+				brasero_device_handle_close (handle);
+				BRASERO_MEDIA_LOG ("Device does not respond");
+				break;
+			}
+
+			sleep (2);
 
 			if (priv->probe_cancelled) {
 				priv->probe = NULL;
@@ -3031,6 +3063,8 @@ brasero_medium_finalize (GObject *object)
 
 	priv = BRASERO_MEDIUM_PRIVATE (object);
 
+	BRASERO_MEDIA_LOG ("Finalizing Medium object");
+
 	if (priv->probe) {
 		priv->probe_cancelled = TRUE;
 		g_thread_join (priv->probe);
@@ -3156,6 +3190,14 @@ brasero_medium_class_init (BraseroMediumClass *klass)
 	                                                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 }
 
+
+/* This function is not public API yet because it was too
+ * late; so use it internally for now. It's mainly for 
+ * convenience.*/
+gboolean
+brasero_drive_can_write_media (BraseroDrive *drive,
+                               BraseroMedia media);
+
 /**
  * brasero_medium_can_be_written:
  * @medium: #BraseroMedium
@@ -3170,59 +3212,13 @@ gboolean
 brasero_medium_can_be_written (BraseroMedium *medium)
 {
 	BraseroMediumPrivate *priv;
-	BraseroDriveCaps caps;
 
 	g_return_val_if_fail (medium != NULL, FALSE);
 	g_return_val_if_fail (BRASERO_IS_MEDIUM (medium), FALSE);
 
 	priv = BRASERO_MEDIUM_PRIVATE (medium);
 
-	if (!(priv->info & BRASERO_MEDIUM_REWRITABLE)
-	&&   (priv->info & BRASERO_MEDIUM_CLOSED))
-		return FALSE;
-
-	if (priv->info & BRASERO_MEDIUM_FILE)
-		return FALSE;
-
-	caps = brasero_drive_get_caps (priv->drive);
-	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_CDR))
-		return (caps & BRASERO_DRIVE_CAPS_CDR) != 0;
-
-	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDR))
-		return (caps & BRASERO_DRIVE_CAPS_DVDR) != 0;
-
-	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDR_PLUS))
-		return (caps & BRASERO_DRIVE_CAPS_DVDR_PLUS) != 0;
-
-	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_CDRW))
-		return (caps & BRASERO_DRIVE_CAPS_CDRW) != 0;
-
-	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW))
-		return (caps & BRASERO_DRIVE_CAPS_DVDRW) != 0;
-
-	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_RESTRICTED))
-		return (caps & BRASERO_DRIVE_CAPS_DVDRW) != 0;
-
-	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_PLUS))
-		return (caps & BRASERO_DRIVE_CAPS_DVDRW_PLUS) != 0;
-
-	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDR_PLUS_DL))
-		return (caps & BRASERO_DRIVE_CAPS_DVDR_PLUS_DL) != 0;
-
-	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_PLUS_DL))
-		return (caps & BRASERO_DRIVE_CAPS_DVDRW_PLUS_DL) != 0;
-
-	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVD_RAM))
-		return (caps & BRASERO_DRIVE_CAPS_DVDRAM) != 0;
-
-	/* All types of BD-R */
-	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_BD|BRASERO_MEDIUM_WRITABLE))
-		return (caps & BRASERO_DRIVE_CAPS_BDR) != 0;
-
-	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_BDRE))
-		return (caps & BRASERO_DRIVE_CAPS_BDRW) != 0;
-
-	return FALSE;
+	return brasero_drive_can_write_media (priv->drive, priv->info);
 }
 
 /**
@@ -3262,6 +3258,45 @@ brasero_medium_can_be_rewritten (BraseroMedium *medium)
 
 	return FALSE;
 }
+/**
+ * brasero_medium_can_use_sao:
+ * @medium: #BraseroMedium
+ *
+ * Gets whether the medium supports SAO.
+ *
+ * Return value: a #gboolean. TRUE if the medium can use SAO write mode , FALSE otherwise.
+ *
+ **/
+gboolean
+brasero_medium_can_use_sao (BraseroMedium *medium)
+{
+	BraseroMediumPrivate *priv;
+
+	g_return_val_if_fail (BRASERO_IS_MEDIUM (medium), FALSE);
+
+	priv = BRASERO_MEDIUM_PRIVATE (medium);
+	return priv->sao;
+}
+
+/**
+ * brasero_medium_can_use_tao:
+ * @medium: #BraseroMedium
+ *
+ * Gets whether the medium supports TAO.
+ *
+ * Return value: a #gboolean. TRUE if the medium can use TAO write mode, FALSE otherwise.
+ *
+ **/
+gboolean
+brasero_medium_can_use_tao (BraseroMedium *medium)
+{
+	BraseroMediumPrivate *priv;
+
+	g_return_val_if_fail (BRASERO_IS_MEDIUM (medium), FALSE);
+
+	priv = BRASERO_MEDIUM_PRIVATE (medium);
+	return priv->tao;
+}
 
 /**
  * brasero_medium_can_use_dummy_for_sao:
