@@ -1005,8 +1005,8 @@ brasero_cdrecord_set_argv (BraseroProcess *process,
 	BraseroBurnResult result;
 	BraseroJobAction action;
 	BraseroBurnFlag flags;
+	gchar *device = NULL;
 	gchar *dev_str;
-	gchar *device;
 
 	cdrecord = BRASERO_CD_RECORD (process);
 	priv = BRASERO_CD_RECORD_PRIVATE (cdrecord);
@@ -1018,13 +1018,11 @@ brasero_cdrecord_set_argv (BraseroProcess *process,
 	g_ptr_array_add (argv, g_strdup ("cdrecord"));
 	g_ptr_array_add (argv, g_strdup ("-v"));
 
-#ifdef HAVE_CAM_LIB_H
-	/* FreeBSD like that better */
+	/* NOTE: that function returns either bus_target_lun or the device path
+	 * according to OSes. Basically it returns bus/target/lun only for FreeBSD
+	 * which is the only OS in need for that. For all others it returns the device
+	 * path. */
 	brasero_job_get_bus_target_lun (BRASERO_JOB (cdrecord), &device);
-#else
-	brasero_job_get_device (BRASERO_JOB (cdrecord), &device);
-#endif
-
 	dev_str = g_strdup_printf ("dev=%s", device);
 	g_ptr_array_add (argv, dev_str);
 	g_free (device);
