@@ -1,27 +1,31 @@
-/***************************************************************************
- *            burn-dvdcss.c
- *
- *  lun aoû 21 14:34:32 2006
- *  Copyright  2006  Rouquier Philippe
- *  bonfire-app@wanadoo.fr
- ***************************************************************************/
-
+/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /*
- *  TBrasero is free software; you can redistribute itand/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ * Libbrasero-burn
+ * Copyright (C) Philippe Rouquier 2005-2009 <bonfire-app@wanadoo.fr>
  *
- *  TBrasero is distributed in the hope that it will be useful
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ * Libbrasero-burn is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to t:
+ * The Libbrasero-burn authors hereby grant permission for non-GPL compatible
+ * GStreamer plugins to be used and distributed together with GStreamer
+ * and Libbrasero-burn. This permission is above and beyond the permissions granted
+ * by the GPL license by which Libbrasero-burn is covered. If you modify this code
+ * you may extend this exception to your version of the code, but you are not
+ * obligated to do so. If you do not wish to do so, delete this exception
+ * statement from your version.
+ * 
+ * Libbrasero-burn is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Library General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to:
  * 	The Free Software Foundation, Inc.,
  * 	51 Franklin Street, Fifth Floor
- * 	Boston, MA  02110-1301, USA.30, Boston, MA 02111-1307, USA.
+ * 	Boston, MA  02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -141,6 +145,7 @@ error_loading:
 static gboolean
 brasero_dvdcss_thread_finished (gpointer data)
 {
+	goffset blocks = 0;
 	gchar *image = NULL;
 	BraseroDvdcss *self = data;
 	BraseroDvdcssPrivate *priv;
@@ -167,6 +172,9 @@ brasero_dvdcss_thread_finished (gpointer data)
 					NULL,
 					BRASERO_IMAGE_FORMAT_BIN);
 
+	brasero_job_get_session_output_size (BRASERO_JOB (self), &blocks, NULL);
+	brasero_track_image_set_block_num (track, blocks);
+
 	brasero_job_add_track (BRASERO_JOB (self), BRASERO_TRACK (track));
 
 	/* It's good practice to unref the track afterwards as we don't need it
