@@ -36,6 +36,11 @@
 
 #include <gtk/gtk.h>
 
+#include "brasero-io.h"
+
+#include "brasero-units.h"
+#include "brasero-medium-monitor.h"
+
 #include "brasero-app.h"
 #include "brasero-utils.h"
 #include "brasero-project.h"
@@ -44,8 +49,6 @@
 #include "brasero-file-chooser.h"
 #include "brasero-uri-container.h"
 #include "brasero-project-type-chooser.h"
-#include "brasero-io.h"
-#include "brasero-medium-monitor.h"
 
 #ifdef BUILD_SEARCH
 #include "brasero-search-beagle.h"
@@ -214,7 +217,7 @@ brasero_project_manager_new_cover_cb (GtkAction *action,
 	GtkWidget *dialog;
 
 	if (manager->priv->type == BRASERO_PROJECT_TYPE_AUDIO) {
-		brasero_project_create_audio_cover (BRASERO_PROJECT (manager->priv->project), edit);
+		brasero_project_create_audio_cover (BRASERO_PROJECT (manager->priv->project));
 		return;
 	}
 
@@ -246,7 +249,8 @@ brasero_project_manager_set_statusbar (BraseroProjectManager *manager,
 	if (!invalid_num && valid_num) {
 		gchar *size_string;
 
-		if (manager->priv->type == BRASERO_PROJECT_TYPE_AUDIO)
+		if (manager->priv->type == BRASERO_PROJECT_TYPE_AUDIO
+		||  manager->priv->type == BRASERO_PROJECT_TYPE_VIDEO)
 			size_string = brasero_units_get_time_string (files_size, TRUE, FALSE);
 		else if (manager->priv->type == BRASERO_PROJECT_TYPE_DATA)
 			size_string = g_format_size_for_display (files_size);
@@ -261,7 +265,8 @@ brasero_project_manager_set_statusbar (BraseroProjectManager *manager,
 	else if (valid_num) {
 		gchar *size_string = NULL;
 
-		if (manager->priv->type == BRASERO_PROJECT_TYPE_AUDIO) {
+		if (manager->priv->type == BRASERO_PROJECT_TYPE_AUDIO
+		||  manager->priv->type == BRASERO_PROJECT_TYPE_VIDEO) {
 			size_string = brasero_units_get_time_string (files_size, TRUE, FALSE);
 			status_string = g_strdup_printf (ngettext ("%d file is supported (%s)", "%d files are supported (%s)", valid_num),
 							 valid_num,
@@ -782,6 +787,11 @@ brasero_project_manager_open_by_mime (BraseroProjectManager *manager,
 				      const gchar *uri,
 				      const gchar *mime)
 {
+	if (!mime) {
+		/* that can happen when the URI could not be identified */
+		return BRASERO_PROJECT_TYPE_INVALID;
+	}
+
 	/* When our files/description of x-brasero mime type is not properly 
 	 * installed, it's returned as application/xml, so check that too. */
 	if (!strcmp (mime, "application/x-brasero")
@@ -882,7 +892,8 @@ brasero_project_manager_open_uri (BraseroProjectManager *manager,
 	}
 
 	uri = g_file_get_uri (file);
-	if (g_file_query_exists (file, NULL)) {
+	if (g_file_query_exists (file, NULL)
+	&& g_file_info_get_content_type (info)) {
 		const gchar *mime;
 
 		mime = g_file_info_get_content_type (info);
